package Whatnew;

=head1 NAME

Whatnew - Generate links of recently documents.

$Id: Whatnew.pm,v 1.7 2002/04/20 13:48:24 T.Shigetome Exp $

=head1 SYNOPSIS

	use Whatnew;
	my($form) = new Whatnew;
	my(%config) = (
		'html_root'	=>	'/home/foo/public_html=http://www.any.com/~foo',
		'search'	=>	'/home/foo/public_html',
		'temp'		=>	'/home/foo/public_html/work/whatnew.tmp',
		'template'	=>	'/home/foo/public_html/work/template.html',
	);
	$form->ParseQuery();
	$form->SetParameter(\%config);
	$form->OutResponse();

=head1 DESCRIPTION

This is for CGI program.
Create links of newest items automatically.
If you update some HTML contents on Web server, it will be automatically appended to links.
And, you can append topics about update to links.
That way is very easy, you just only describe about update information in that HTML file.
Update information is described with meta tag in head block.

Like as...

	<META name="UpdateInfo" content="
		Second update information
		Also you can describe more than two lines.
	_ThIs_Is_A_BoUnDaRy_StRiNg_
		First update information
	_ThIs_Is_A_BoUnDaRy_StRiNg_
		It's NEW!!
	_ThIs_Is_A_BoUnDaRy_StRiNg_
	">

Above `_ThIs_Is_A_BoUnDaRy_StRiNg_' is used for boundary string of each update information record.

=head2 CGI ARGUMENT

This module will be used for CGI program.
Like as...

	http://www.any.com/~foo/cgi-bin/sample.cgi?config=test&func=disp

=over 4

=item func

There are two functions.

=over 4

=item disp

Response links of newest items as HTML page.
Usually use this function.

=item get

Response newest items record file.
This is used for join another server information.
This function will be used described in parameter of `C<remote>'.

=back

=item config

Value of `config' in QUERY STRING.

That value is used for hint of selecting from parameter set.
I hope to use with F<Params.pm>.

=back

=head2 METHOD

=over 4

=item new([$cgi_object])

Create new instance

	my($form) = new Whatnew;

The CGI instance will be created by Whatnew automatically.

or

	my($q) = new CGI;
	my($form) = new Whatnew($q);


=item SetParameter($rh_para)

Setup of parameter
The $rh_para is reference of hash or filename of parameter file.
If you specify a parameter file name, that parameter file will be parsed with F<Params.pm>.

	my($form) = new Whatnew($q);
	my(%config) = (
		'html_root'	=>	'file path=url path',
		'search'	=>	'search file path',
		'temp'		=>	'temporary file',
		'template'	=>	'HTML template',
		'remote'	=>	'http://www.another.server/cgi-bin/sample.cgi?config=configname&func=get',
		'hidden'	=>	'hidden path',
		'mask'		=>	'regular expression for mask',
	);
	$form->SetParameter(\%config);

=over 4

=item html_root

Specify file path connection with URL path.
If you want to two or more html root, you can describe separated each one by `;'.
Do NOT insert space character between each define.

=item search

Scan file path of newest items.
This module will search all of items under specified path.
If you want to two or more, you can describe separated each one by `;'.
Do NOT insert space character between each define.

=item temp

Temporary file.
This module will generate temporary file for performance.

=item template

Used for links page.
It's easy customizing.
Like as...

	<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN">
	<HTML lang="en">
	<HEAD>
	<META http-equiv="Content-Type" CONTENT="text/html; charset=ISO-8859-5">
	<META name="Author" content="Foo">
	<TITLE>What's NEW?</TITLE>
	<MEATA name="UpdateInfo" content="
		What's NEW template
	_ThIs_Is_A_BoUnDaRy_StRiNg_"
	>
	</HEAD>
	<BODY>
	<H1>What's NEW?</H1>
	<HR>
	<TABLE summary="NEW contents" rule="none" cellspacing="0">
	<!--
		Following comment statement will be replaced to new documents information.
		You can customize that.
		These keywords will be handled about associated document.
		Like as...

		%mtime%	: Modified time
		%URL%	: URL
		%title%	: Title in head block of that document
		%x-meta-updateinfo%
			: Update information
			  It's described in head block of that document.
			  If you want to use this field, you must append it along specified format when you update document.
	-->
	<!-- NEW items
	<TR>
	<TD VALIGN="top">%mtime%</TD><TD><P><A HREF="%URL%">%title%</A></P><PRE>%x-meta-updateinfo%</PRE></TD>
	</TR>
	NEW items -->
	</TABLE>
	<HR>
	</BODY>
	</HTML>

=item remote

Optional parameter.
Specify another Whatnew CGI program with `C<func>=C<get>'.
If you have several server, you can merge to one links page from each server.
If you want to two or more, you can describe separated each one by `;'.
Do NOT insert space character between each define.

=item hidden

Optional parameter.
Specify hidden path.
That path and under items will be ignored.
If you want to two or more, you can describe separated each one by `;'.
Do NOT insert space character between each define.

There is another way to hide.
Describe `NOPICK' as update information.
That content will be ignored.
Basically you should not use this way, it is heavy.

=item mask

Optional parameter.
Specify mask.
If full path filename is matched to mask, that will be ignored.
If you want to two or more, you can describe separated each one by `;'.
Do NOT insert space character between each define.

=back

Above delimiter string that is `;' is defined by `C<$Params::delimiter>' in F<Params.pm>.

=item OutResponse()

Output response of newest items link along specified function by`C<func>'.

=item config()

Return configuration name of `config' from CGI query string.
I think, that configuration name is useful with F<Params.pm>. ;)

Like as...
sample.cgi 

	use Whatnew;
	my($form)= new Whatnew;
	$form->ParseQuery();
	local($main::configname) = $form->config();
	$form->SetParameter("sample.par");
	$form->OutResponse();

sample.par

	########################################################
	#if $main::configname eq 'test'
		#define CONFIG_NAME test
	#endif
	#define ROOTPATH /home/foo/public_html
	#define	WORKPATH %ROOTPATH%/work
	#define HTML_ROOT http://www.any.com/~foo
	# 12*60*60
	#
	# define config1 parameters
	#
	#
	# File path to HTML root
	#
	%CONFIG_NAME% html_root{
		%ROOTPATH%=%HTML_ROOT%;
	};
	#
	# temporary file
	#
	%CONFIG_NAME% temp{
		%WORKPATH%/whatnew.tmp;
	};
	#
	# HTML template
	#
	%CONFIG_NAME% template{
		%WORKPATH%/template.html;
	};
	#
	# Common update information
	#
	%CONFIG_NAME% search{
		%ROOTPATH%;
	};
	#
	# No care directries
	#
	%CONFIG_NAME% hidden{
		%ROOTPATH%/debug;
	};
	%CONFIG_NAME%	mask{
		template\.html;
		test;
	};

=item charset($ocode)

$ocode is to give to jcode::covert(). 
If your contents is used some code, it will be cause of corrupt.
This to do is made clear that problem.

=item debug($debug_opt)

Toggle each debug condition.
Default is debug off.
There are four debugging options.

	$form->debug('configpara');
	$form->debug('remote');
	$form->debug('search');
	$form->debug('template');

Message of debugging will be outputted to STDERR.
Also. you can log by redirect STDERR with F<debuglog.pl> or other.

When you terminate debugging, you can strip debugging code from this module.
All of debugging code are between  line of `#>>>DEBUG_CODE'.
These are able to strip by F<striper.pl>.

=over 4

=item configpara

Debug of parameter

=item remote

Debug of getting records via HTTP.

=item search

Debug of search process.

=item template

Debug of parsing template.

=back

=back

=head2 Require modules

F<strict.pm>, 
F<Carp.pm>, 
F<CGI.pm>, 
F<HTML::HeadParser.pm>, 
F<Params.pm>, 
F<Timestr.pm>, 

Following modules are used for get from remote.
So, if you want to get update information from other server, you must use these modules yourself.

F<LWP::UserAgent>, 
F<HTTP::Request>, 
F<HTTP::Message>,

=head1 SEE ALSO

F<Params.pm>, F<Timestr.pm>, F<CGI.pm>

=head1 AUTHOR

Tetsuya Shigetome, E<lt>t_shigetome@muf.biglobe.ne.jpE<gt>.

=head1 HISTORY

=over 4

=item $Revision: 1.7 $

$Log: Whatnew.pm,v $
Revision 1.7  2002/04/20 13:48:24  T.Shigetome
Compliant

History.pm

Some variable is globalised.


=item Revision: 1.6 

Log: Whatnew.pm,v 
Revision 1.6  2002/04/14 15:25:03  T.Shigetome

Fixed bugs

o Sort order is not numerically.

Compliant Timestr.pm and Params.pm.
I hope, it's given simplified to use parameter file.


=item Revision: 1.5 

Log: Whatnew.pm,v 
Revision 1.5  2001/07/30 09:28:33  shige


 o Fixed problem of Server error 500 when get from remote.

Revision 1.4  2001/05/19 16:54:03  shige

 o Correct corrupt problem when comment avail after replace keyword.


=item Revision: 1.3

Log: Whatnew.pm,v 
Revision 1.3  2001/02/10 16:32:34  shige

o Change some comments.


=item Revision: 1.2

Log: Whatnew.pm,v 
Revision 1.2  2001/02/10 16:02:07  shige

 o Correct merge display problem.
 o Change to sepcify debug options.
 o Some new capables
    Support convert code.
    Support hide temporary.

=item Revision: 1.1

Log: Whatnew.pm,v 
Revision 1.1  2000/12/14 16:59:34  shige

First of Perl module version

=back

=cut

$Whatnew::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);

use strict;
use Carp;
use CGI;
use HTML::HeadParser;
use Params;
use Timestr;
#use RefHandler;
#>>>NO_REMOTE
#use LWP::UserAgent;
#use HTTP::Request;
#require HTTP::Message;
#use LWP::Debug qw(+);
#>>>NO_REMOTE
@Whatnew::fieldlist = ('mtime','URL','title','x-meta-updateinfo');
%Whatnew::BoundaryString = (
	'record'	=> '_ThIs_Is_A_BoUnDaRy_StRiNg_',
	'field'		=>	'_DaTa_SePaRaToR_',
);
%Whatnew::HeaderField = (
	'title'				=>	\&GetTitle,
	'x-meta-updateinfo'	=>	\&UpdateInfoParser
);
%Whatnew::ReplaceKeys = (
	'<!-- NEW items\s+(.+)NEW items -->'	=>	\&NewItemsExtender,
);
######################
# Constructer
sub new
{
my($pkg,$cgi_query) = @_;
my($rh_this);
	$rh_this =  bless {
		'config'		=>	undef,
#>>>DEBUG_CODE
		'_debug'			=>	{
			'configpara'	=>	undef,
			'search'		=>	undef,
			'template'		=>	undef,
			'remote'		=>	undef,
		},# 'remote', 'configpara '
#>>>DEBUG_CODE
		'_DependentFile'	=>	{
			'_file2urlpath'	=>	{},
			'_NewestPeriod'	=>	30,		# unit is day 
			'_Suffix'		=>	['\.html','\.htm'],
			'_HtmlRoot'		=>	undef, #{},
			'_SuffixMatch'	=>	undef, #'',
			'_SearchPath'	=>	undef, #[],
			'_HiddenPath'	=>	undef, #[],
			'_HiddenMatch'	=>	undef, #'',
			'_Mask'			=>	undef, #[],
			'_MaskMatch'	=>	undef, #'',
			'_NoPickWord'	=>	'NOPICK',
		},
		'_UpdateInfo'		=>	{
			'Remote'	=>	undef, #[],
			'Local'		=>	[],
			'HeaderField'	=>	\%Whatnew::HeaderField,
		},
		'_TemaplateFile'	=>	{
			'filename'		=>	undef,
			'_ReplaceKeys'	=>	\%Whatnew::ReplaceKeys,
		},
		'_Temporary'	=>	{
			'filename'		=>	undef,
			'lifetime'	=>	12,	# unit is hour
			'fieldlist'	=>	\@Whatnew::fieldlist,
			'mtime'		=>	undef,
			'remote'	=>	undef, #[],
		},
		'_BoundaryString'	=>	\%Whatnew::BoundaryString,
		'_ParameterEvaluater'	=>	undef,#{},	# It' described later
		'_Method'	=>	{
			'_Response'	=>	undef,
		},
		'_CgiQuery'		=>	$cgi_query,
		'charset'		=>	undef,	# If there are mixed charset, you can specify code. But, jcode.pl require yourself.
	}, $pkg;
#######################
# Defined CGI argument evaluation subroutine
#
	$rh_this->{'_ParameterEvaluater'} = {
#//////////////////////////////////////////
		'config' => sub {
		my($pkg,$string) = @_;
			$pkg->{'config'} = $string;
		},
#//////////////////////////////////////////
		'func' => sub {
		my($pkg,$string) = @_;
		my(%ResponseMethod) = (
			'disp'	=>	\&OutWhatNew,
			'get'	=>	\&OutPlainInfo,
		);
			# select Response method
			die "Unknown funtion : $string" unless ( defined($ResponseMethod{$string}) );
			$pkg->{'_Method'}{'_Response'} = $ResponseMethod{$string};
		},
	};

	return $rh_this;
}
#######################
# Method autoload
sub AUTOLOAD
{
my($pkg) = shift;
my($type) = ref($pkg) || croak "$pkg is not an object";
my($name) = $Whatnew::AUTOLOAD;
	$name =~ s/.*://;
	unless( exists($pkg->{$name}) )
	{
#>>>DEBUG_CODE
		unless( $name eq 'debug' )
		{
#>>>DEBUG_CODE
			croak "Can't access `$name' field in object of class $type";
#>>>DEBUG_CODE
		}
		else
		{
			if( @_ )
			{
				foreach( @_ )
				{
					if( defined($pkg->{'_debug'}{$_}) )
					{
						delete $pkg->{'_debug'}{$_};
					}
					else
					{
						$pkg->{'_debug'}{$_} = 1;
					}
				}
			}
		}
		return wantarray ? keys(%{$pkg->{'_debug'}}) : $pkg->{'_debug'};
#>>>DEBUG_CODE
	}

	return $pkg->{$name} = shift if( @_ );
	return $pkg->{$name};
}
#	It's a scapegoat for AUTOLOAD.
sub	DESTROY{}
#######################
# Parse Query string
sub ParseQuery
{
my($pkg) = shift;
local($_);
my($key,$func,$value);
	unless( defined($pkg->{'_CgiQuery'}) )
	{
		$pkg->{'_CgiQuery'} = new CGI;
	}
#>>>DEBUG_CODE
	else
	{
		warn if defined($pkg->{'_debug'}{'configpara'});
	}
#>>>DEBUG_CODE

	foreach $key ( ($pkg->{'_CgiQuery'}->param) )
	{
		$value = $pkg->{'_CgiQuery'}->param($key);
#>>>DEBUG_CODE
		warn "$key=$value" if defined($pkg->{'_debug'}{'configpara'});
#>>>DEBUG_CODE
		if( defined($pkg->{'_ParameterEvaluater'}{$key}) )
		{
			&{$pkg->{'_ParameterEvaluater'}{$key}}($pkg,$value);
		}
		else
		{# It may be extended parameter. But do NOT override.
			$pkg->{$key} = $value unless exists($pkg->{$key});
		}
	}
}
#######################
# Set parameter
sub SetParameter
{
my($pkg) = shift @_;
my($rh_para) = @_;
my(%para_parser) = (
	'html_root'	=>	[\&Set2HashRef,		\$pkg->{'_DependentFile'}{'_HtmlRoot'},		qq/Unknown "html_root" information type /],
	'search'	=>	[\&Set2ArrayRef,	\$pkg->{'_DependentFile'}{'_SearchPath'},	qq/Unknown "search" argument type /],
	'temp'		=>	[\&Set2ScalarRef,	\$pkg->{'_Temporary'}{'filename'},			qq/Unknown "temporary" argument type /],
	'template'	=>	[\&Set2ScalarRef,	\$pkg->{'_TemaplateFile'}{'filename'},		qq/Unknown "template" argument type /],
	'hidden'	=>	[\&Set2ArrayRef,	\$pkg->{'_DependentFile'}{'_HiddenPath'},	qq/Uknown "hidden" argument type /],
	'mask'		=>	[\&Set2ArrayRef,	\$pkg->{'_DependentFile'}{'_Mask'},			qq/Unknown "mask" argument type /],
	'remote'	=>	[\&Set2ArrayRef,	\$pkg->{'_Temporary'}{'remote'},			qq/Unknown "remote" argument type /],
	);
	unless( ref($rh_para) )
	{# It's specifi paramter file
		my($para) = new Params($rh_para);
		my($configname) = $pkg->config();
		warn "Handled as parameter file $rh_para." if defined($pkg->{'_debug'}{'configpara'});
		croak "Do not specify configuration name in CGI query string" unless defined($configname);
		$rh_para = $para->GetData($configname);
	}
	foreach( (keys(%para_parser)) )
	{
		if( defined( $rh_para->{$_} ) )
		{
#>>>DEBUG_CODE
			warn "Parse parameter : $_ => ${$rh_para}{$_}" if defined($pkg->{'_debug'}{'configpara'});
#>>>DEBUG_CODE
			die "$para_parser{$_}[2] $pkg" unless &{$para_parser{$_}[0]}($para_parser{$_}[1],$rh_para->{$_});
		}
#>>>DEBUG_CODE
		else
		{
			warn "Ignore parameter : $_" if defined($pkg->{'_debug'}{'configpara'});
		}
#>>>DEBUG_CODE
	}
}
#######################
# Output Response
sub OutResponse
{
my($pkg) = shift;
	die "Unknown method of output response : $pkg" unless defined($pkg->{'_Method'}{'_Response'});
	&{$pkg->{'_Method'}{'_Response'}}($pkg);
}
#######################
# Update Temporary file
sub UpdateTemporary
{
my($pkg) = shift;
	die "Do NOT specify temporary file $pkg." unless defined($pkg->{'_Temporary'}{'filename'});
	die "Do NOT specify search path $pkg." unless defined($pkg->{'_DependentFile'}{'_SearchPath'});

	# Check out temporary life time
	$pkg->{'_Temporary'}{'mtime'} = (stat($pkg->{'_Temporary'}{'filename'}))[9];
	if( !defined($pkg->{'_Temporary'}{'mtime'}) ||  -M $pkg->{'_Temporary'}{'filename'} > ($pkg->{'_Temporary'}{'lifetime'}/24) )
	{# Browse search path
		my(@newfiles);
		$pkg->{'_Temporary'}{'mtime'} = $^T;
		@newfiles = $pkg->GetNewFiles();
		# Get each file field information
		$pkg->CreateLocalInfo(\@newfiles,$pkg->{'_UpdateInfo'}{'Local'});
		{# Update temporary
			open(TEMP,">$pkg->{'_Temporary'}{'filename'}") || die "Can not create temporary file $pkg->{'_Temporary'}{'filename'}: $!";
			for( @{$pkg->{'_UpdateInfo'}{'Local'}} )
			{
				print TEMP $_.$pkg->{'_BoundaryString'}{'record'}."\n";
			}
			close(TEMP);
		}
	}
	else
	{# Load temporary
		my($buffer);
		open(TEMP,"$pkg->{'_Temporary'}{'filename'}") || die "Can not open temporary file $pkg->{'_Temporary'}{'filename'}: $!";
		while(<TEMP>)
		{
			$buffer .= $_;
		}
		close(TEMP);
		@{$pkg->{'_UpdateInfo'}{'Local'}} = split(/$pkg->{'_BoundaryString'}{'record'}\n*/,$buffer);
	}

}
#######################
# Output What's NEW page as response
sub OutWhatNew
{
my($pkg) = shift;
my($lastmodify,$expire,$max_age);
my($replace_key);
my($content);
	$pkg->UpdateTemporary();
	$pkg->GetRemote() if defined($pkg->{'_Temporary'}{'remote'});

	open(FORM,"$pkg->{'_TemaplateFile'}{'filename'}")|| die "Can not open template file  $pkg->{'_TemaplateFile'}{'filename'}: $!";

	$lastmodify = &LocalToGmtDate($pkg->{'_Temporary'}{'mtime'});
	$expire = &LocalToGmtDate( $pkg->{'_Temporary'}{'mtime'} + ($pkg->{'_Temporary'}{'lifetime'}*60*60) );
	$max_age = ($pkg->{'_Temporary'}{'mtime'} + ($pkg->{'_Temporary'}{'lifetime'}*60*60)) - $^T ;

	# Load template file
	while( <FORM> )
	{
		$content .= $_;
	}
	foreach( keys(%{$pkg->{'_TemaplateFile'}{'_ReplaceKeys'}}) )
	{
		warn "Can NOT find replacekey in template : $_"
			unless $content =~ s/$_/{&{$pkg->{'_TemaplateFile'}{'_ReplaceKeys'}{$_}}($pkg,$1);}/sge;
	}
	print <<"__HTTP_HEADER__";
Content-type: text/html
Cache-Control: max-age = $max_age
Last-Modified: $lastmodify
Expires: $expire

$content
__HTTP_HEADER__
	return;
}
#######################
# Output Plain text information as response
sub OutPlainInfo
{
my($pkg) = shift;
my($length,$lastmodify,$expire,$max_age,$entity);
my($nameas);

	$pkg->UpdateTemporary();

	$lastmodify = &LocalToGmtDate($pkg->{'_Temporary'}{'mtime'});
	$expire = &LocalToGmtDate($pkg->{'_Temporary'}{'mtime'} + ($pkg->{'_Temporary'}{'lifetime'}*60*60));
	$max_age = ($pkg->{'_Temporary'}{'mtime'} + ($pkg->{'_Temporary'}{'lifetime'}*60*60)) - $^T ;

	foreach(@{$pkg->{'_UpdateInfo'}{'Local'}})
	{
		s/\n*$//s;
		$entity .= $_.$pkg->{'_BoundaryString'}{'record'}."\n";
	}
	$length = length($entity);

	# Specify filename
	$nameas = $pkg->{'_CgiQuery'}->param('nameas');
	if( defined($nameas) && $nameas ne '' )
	{
		$nameas = '; name="'.$nameas.'"';
	}

	print <<"__HTTP_HEADER__";
Content-Type: application/octet-stream $nameas
Content-Length: $length
Cache-Control: max-age = $max_age
Last-Modified: $lastmodify
Expires: $expire

$entity
__HTTP_HEADER__
}
#######################
# Get New file names
sub GetNewFiles
{
my($pkg) = shift;
my(@file_list);

	die "Do NOT specify search path : $pkg" unless defined($pkg->{'_DependentFile'}{'_SearchPath'});

	$pkg->{'_DependentFile'}{'_SuffixMatch'} = join('|',@{$pkg->{'_DependentFile'}{'_Suffix'}}) if defined($pkg->{'_DependentFile'}{'_Suffix'});
	$pkg->{'_DependentFile'}{'_HiddenMatch'} = join('|',@{$pkg->{'_DependentFile'}{'_HiddenPath'}}) if defined($pkg->{'_DependentFile'}{'_HiddenPath'});
	$pkg->{'_DependentFile'}{'_MaskMatch'} = join('|',@{$pkg->{'_DependentFile'}{'_Mask'}}) if defined($pkg->{'_DependentFile'}{'_Mask'});

	foreach(@{$pkg->{'_DependentFile'}{'_SearchPath'}})
	{
#>>>DEBUG_CODE
warn $_ if defined($pkg->{'_debug'}{'configpara'});
#>>>DEBUG_CODE
		# make file list
		$pkg->GetNewFilesCurrentPath($_,\@file_list);
	}
#>>>DEBUG_CODE
warn $#file_list if defined($pkg->{'_debug'}{'search'});
#>>>DEBUG_CODE
	return	 @file_list;
}
#######################
# Get New file names
sub GetNewFilesCurrentPath
{
my($pkg) = shift;
my($current_path,$ra_filelist) = @_;
my(@files);
my($mtime);

#######################
	opendir(DIR,$current_path) || die "Can't open search path $current_path: $!";
	@files = grep (!/^\.\.{0,1}$/ , readdir(DIR));
	closedir(DIR);
#>>>DEBUG_CODE
warn  $#files if defined($pkg->{'_debug'}{'search'});
#>>>DEBUG_CODE
	@files = grep (!/($pkg->{'_DependentFile'}{'_MaskMatch'})/o , @files) if defined($pkg->{'_DependentFile'}{'_MaskMatch'});
#>>>DEBUG_CODE
warn "$#files : $pkg->{'_DependentFile'}{'_MaskMatch'}" if defined($pkg->{'_debug'}{'search'});
#>>>DEBUG_CODE
	foreach( @files )
	{
		next if /^$/;
		$_ = "$current_path/$_";
		next if m/^($pkg->{'_DependentFile'}{'_HiddenMatch'})$/o;

		($mtime) = (stat($_))[9];
		if( -d _ )
		{
			$pkg->GetNewFilesCurrentPath($_,$ra_filelist);
		}
		else
		{
			if( m/($pkg->{'_DependentFile'}{'_SuffixMatch'})$/o &&  -M _ < $pkg->{'_DependentFile'}{'_NewestPeriod'} )
			{
				push(@{$ra_filelist},join($pkg->{'_BoundaryString'}{'field'},$mtime+$Timestr::gmadjust,$_));
			}
		}
	}
#>>>DEBUG_CODE
warn "$current_path : $#{$ra_filelist}" if defined($pkg->{'_debug'}{'search'});
#>>>DEBUG_CODE
	return $ra_filelist
}
#######################
# Create information
sub CreateLocalInfo
{
my($pkg) = shift;
my($ra_files,$ra_buffer) = @_;
my($mtime,$filename);
my(@local_info);
my($buffer);
my($h);
my($key,$value,$obj);
my(%hash_buffer);
my($path_expr) = join('|',keys(%{$pkg->{'_DependentFile'}{'_HtmlRoot'}}));
	foreach(@{$ra_files})
	{
		undef %hash_buffer;
		undef $h;
		$h = new HTML::HeadParser;
		($hash_buffer{'mtime'},$filename) = split($pkg->{'_BoundaryString'}{'field'},$_);

		# Convert file path to URL
		$hash_buffer{'URL'} = $filename;
		$hash_buffer{'URL'} =~ s/^($path_expr)/$pkg->{'_DependentFile'}{'_HtmlRoot'}{$1}/o;

		open(TEMP,$filename) || die "Can Not open target file $filename: $!";
		undef $buffer;
		while(<TEMP>)
		{
			$buffer .= $_;
		}
		$h->parse($buffer);
		while( ($key,$value) = each(%{$pkg->{'_UpdateInfo'}{'HeaderField'}}))
		{
			if( ref($value) eq 'CODE' )
			{
				$hash_buffer{$key} = &{$value}($pkg,\%hash_buffer,$h->header($key));
			}
			elsif( ref($value) eq '' )
			{
				$hash_buffer{$key} = $h->header($key);
			}
			else
			{
				die "Unknown Header field parser define : $key => $value ";
			}
		}
		unless( $hash_buffer{'x-meta-updateinfo'} eq $pkg->{'_DependentFile'}{'_NoPickWord'} )
		{# Push record string
			my(@work);
			@work = ();
			foreach( @{$pkg->{'_Temporary'}{'fieldlist'}} )
			{
				push(@work,$hash_buffer{$_});
			}
			push(@{$ra_buffer},join($pkg->{'_BoundaryString'}{'field'},@work));
		}
	}
	return	 $ra_buffer;
}
#######################
# Title get
sub GetTitle
{
my($pkg,$rh_buffer,$string) = @_;
	if( defined($string) && $string ne '' )
	{
		if( defined($pkg->{'charset'}) )
		{
			local($Whatnew::work) = $string;
			# convert charset to specify code
			&jcode'convert(*Whatnew'work,$pkg->{'charset'});
			return $Whatnew::work;
		}
		else
		{
			return $string;
		}
	}
	else
	{
		return $rh_buffer->{'URL'};
	}
}
#######################
# Update information parser
sub UpdateInfoParser
{
my($pkg,$rh_buffer,$string) = @_;
	if( defined($string) )
	{
		if( defined($pkg->{'charset'}) )
		{
			local($Whatnew::work) = (split(/$pkg->{'_BoundaryString'}{'record'}\n*/,$string))[0];
			# convert charset to specify code
			&jcode'convert(*Whatnew'work,$pkg->{'charset'});
			return $Whatnew::work;
		}
		else
		{
			return (split(/$pkg->{'_BoundaryString'}{'record'}\n*/,$string))[0] ;
		}
	}
	else
	{
		return ' ';
	}
}
#######################
# Field extender 2nd
sub NewItemsExtender
{
my($pkg) = shift;
my($base_string) = @_;
my(@updateinfo) = @{$pkg->{'_UpdateInfo'}{'Local'}};
my(@record,%h_record);
my($i,$sprintwork,$work);
my($replace_key,$ra_format);
my($extended_string) = '';

#>>>DEBUG_CODE
	warn $base_string if defined($pkg->{'_debug'}{'template'});
#>>>DEBUG_CODE
	if( defined($pkg->{'_UpdateInfo'}{'Remote'}) )
	{
		push(@updateinfo,@{$pkg->{'_UpdateInfo'}{'Remote'}});
	}
	@updateinfo =  reverse sort {my($a_time,$b_time);$a =~ m/^\d+/; $a_time = $&;$b =~ m/^\d+/; $b_time = $&; $a_time <=> $b_time;} @updateinfo;

	foreach( @updateinfo )
	{
		next if m/^\s*$/;
		undef %h_record;
		$pkg->SplitRecord($_,\%h_record);
		$h_record{'mtime'} = &GetTimeStr($h_record{'mtime'});

		$sprintwork = $base_string;
		$sprintwork =~ s/\%(mtime|URL|title|x-meta-updateinfo)\%/$h_record{$1}/sg;
		$extended_string .= $sprintwork;
	}
	return $extended_string;
}
########################################################
# Get HASH ref array of new record
sub GetHASHArrayOfNewItems
{
my($pkg) = shift;
my($ra_array) = @_;
my(%h_record);

	$pkg->UpdateTemporary();
	$pkg->GetRemote() if defined($pkg->{'_Temporary'}{'remote'});

	@{$ra_array} = @{$pkg->{'_UpdateInfo'}{'Local'}};
	if( defined($pkg->{'_UpdateInfo'}{'Remote'}) )
	{
		push(@{$ra_array},);
	}
	@{$ra_array} =  reverse sort {my($a_time,$b_time);$a =~ m/^\d+/; $a_time = $&;$b =~ m/^\d+/; $b_time = $&; $a_time <=> $b_time;} @{$ra_array};

	foreach( @{$ra_array} )
	{
		next if m/^$/;
		undef %h_record;
		$pkg->SplitRecord($_,\%h_record);
		%{$_} = %h_record;
	}
}
########################################################
# Split record
sub SplitRecord
{
my($pkg) = shift;
my($record,$rh_buffer) = @_;
my(@record,$i);
my(%buffer);
	@record = split(/$pkg->{'_BoundaryString'}{'field'}/,$record);
	$i = 0;
	foreach( @{$pkg->{'_Temporary'}{'fieldlist'}} )
	{
		$rh_buffer->{$_} = $record[$i];
		++$i;
	}
}
#>>>NO_REMOTE
########################################################
# Get What'NEW information from remote server
sub GetRemote
{
my($pkg) = shift;
my($header) = @_;
my($ua) = LWP::UserAgent->new;
my($request,$response);

	$header = new HTTP::Headers unless( defined($header) );
	$header->referer($ENV{'HTTP_REFERER'}) if defined($ENV{'HTTP_REFERER'});
	$header->referer($ENV{'HTTP_USER_AGENT'}) if defined($ENV{'HTTP_USER_AGENT'});
	$header->header(Accept => "*/*");
#>>>DEBUG_CODE
	warn $header->as_string() if defined($pkg->{'_debug'}{'remote'});
#>>>DEBUG_CODE
	foreach( @{$pkg->{'_Temporary'}{'remote'}} )
	{
		next if /^$/;
#>>>DEBUG_CODE
		warn "Remote URL : $_" if defined($pkg->{'_debug'}{'remote'});
#>>>DEBUG_CODE
		$request = HTTP::Request->new(GET => $_, $header);
		warn if defined($pkg->{'_debug'}{'remote'});

		$response = $ua->request($request);


		if( $response->is_success )
		{
#>>>DEBUG_CODE
warn $response->{'_content'} if defined($pkg->{'_debug'}{'remote'});
#>>>DEBUG_CODE
			push(@{$pkg->{'_UpdateInfo'}{'Remote'}},split(/$pkg->{'_BoundaryString'}{'record'}\n*/,$response->{'_content'}) );
		}
#>>>DEBUG_CODE
		else
		{
			warn $response->status_line." : $_" if defined($pkg->{'_debug'}{'remote'});
		}
#>>>DEBUG_CODE
	}
}
#>>>NO_REMOTE
1;
